' Caps Lock Notifier Installer utility to finalize the installation package and
' add the languages French, German, and Italian to the user interface.
'
' The MSI package generated by Visual Studio is not immediately serviceable.
' In a first step, this script applies the transformations contained in the file
' "Fix.mst" to the installer package.
' This makes a usable, English only installation package.
' The following is a list of transformations applied with "Fix.mst".
'
' * In the CustomAction table, in the row with Action =
'   _7623E591_0B8C_4429_861F_B24C1A413004, the Type is set to 1.
'   This allows the custom action that closes "Caps Lock Notifier.exe" when
'   uninstalling or reinstalling to run immediately.
'
' * In the CustomAction table, in the row with Action =
'   _CEEA65B8_9B68_4D1B_8999_2064AD6D1BEE, the Type is set to 3282.
'   This instructs the installer to run "Caps Lock Notifier.exe" asynchronously.
'
' * In the InstallExecuteSequence table, in the row with Action =
'   _7623E591_0B8C_4429_861F_B24C1A413004, the Condition is set to
'   '(REMOVE~="ALL" OR REINSTALL~="ALL") AND ProductState<>1' (without quotes)
'   and the Sequence is set to 1399.
'   This allows the immediate custom action that closes Caps Lock Notifier
'   to run before validation when uninstalling or reinstalling.
'
' In the second step, in the File table, the Language is set to 0 where missing
' (the row's primary key is not the same for all configurations).
' This is not strictly necessary, but it allows the package to pass ICE60
' validation.
'
' In the third step, the script embeds the transform files "fr-FR.mst",
' "de-DE.mst", and "it-IT.mst" with the names 1036, 1031, and 1040 respectively
' in the MSI package.
' These are the language codes for French (France), German (Germany), and
' Italian (Italy) respectively.
' The transform files add support for these languages.
' Each file contains modified versions of language-specific values in the
' database.
' Translations of non-predefined texts are listed below.
'
' English: "[ProductName] requires Windows XP or later."
' French:  "[ProductName] ncessite Windows XP ou une version ultrieure."
' German:  "[ProductName] erfordert Windows XP oder hher."
' Italian: "[ProductName] richiede Windows XP o versioni successive."
'
' English: 
' [ProductName] is copyrighted by [Manufacturer]. This software is provided as
' it is with no warranty of any kind. You are welcome to redistribute it.
'
' French:
' [ProductName] est protg par [Manufacturer]. Ce logiciel est fourni en l'tat
' sans garantie d'aucune sorte. Vous tes invit  le redistribuer.
'
' German:
' [ProductName] ist urheberrechtlich geschtzt von [Manufacturer]. Diese
' Software wird so wie sie ist zur Verfgung gestellt, ohne Garantie jeglicher
' Art. Sie knnen sie gerne weiter verteilen.
'
' Italian:
' [ProductName]  protetto da copyright da [Manufacturer]. Questo software viene
' fornito cos com' senza garanzia di alcun tipo. Si  liberi di
' ridistribuirlo.
'
' In the last step, the language in the summary information is set to
' "1033,1036,1031,1040" (without double quotes).
' These are the comma-separated language codes for English (United States),
' French (France), German (Germany), and Italian (Italy) respectively.
' This change enables the specified languages for the installer user interface.
'
' Note than the installer languages also include the specification of a
' sublanguage (United States for English, France for French, etc.), while Caps
' Lock Notifier only supports generic primary languages.

Option Explicit

Const msiOpenDatabaseModeTransact = 1

Const msiViewModifyAssign         = 3

' Check argument count, and display help if argument not present or too many
' arguments or if any argument contains a question mark.
Dim argCount : argCount = Wscript.Arguments.Count
If argCount > 2 Then argCount = 0
If argCount > 1 Then If InStr(1, Wscript.Arguments(1), "?", vbTextCompare) > 0 _
 Then argCount = 0
If argCount > 0 Then If InStr(1, Wscript.Arguments(0), "?", vbTextCompare) > 0 _
 Then argCount = 0
If (argCount = 0) Then
	Wscript.Echo _
	 "Multilanguage update tool for Caps Lock Notifier Installer" & _
	 vbNewLine & vbNewLine & _
	 "1st argument is the path to MSI database (installer package)." & _
	 vbNewLine & _
	 "2nd argument is the directory with the transform files (.mst)." _
	 & vbNewLine & _
	 "If the 2nd argument is missing, the current directory will be used."
	Wscript.Quit 1
End If

' Evaluate command-line arguments
Dim databasePath : databasePath = Wscript.Arguments(0)
Dim importDir
If argCount > 1 Then
	importDir = Wscript.Arguments(1)
	' Make sure the import path ends with a slash or backslash.
	Dim lastChar
	lastChar = Right(importDir, 1)
	If lastChar <> "/" And lastChar <> "\" Then importDir = importDir & "\"
Else
	importDir = "" ' Fall back to current directory.
End if

Dim installer : Set installer = Nothing
On Error Resume Next
' Update the installer package database.
UpdateDatabase
CheckError

Function AddRecord(view, filename, storageName)
	Dim record : Set record = installer.CreateRecord(2)
	record.StringData(1) = storageName
	view.Execute record
	' Insert storage - copy data into stream.
	record.SetStream 2, importDir & fileName
	view.Modify msiViewModifyAssign, record
End Function

Sub CheckError
	Dim message, errRec
	If Err = 0 Then Exit Sub
	message = Err.Source & " " & Hex(Err) & ": " & Err.Description
	If Not installer Is Nothing Then
		Set errRec = installer.LastErrorRecord
		If Not errRec Is Nothing Then _
		 message = message & vbNewLine & errRec.FormatText
	End If
	Wscript.Echo message
	Wscript.Quit 2
End Sub

Sub UpdateDatabase
	' Connect to Windows Installer object.
	Set installer = Wscript.CreateObject("WindowsInstaller.Installer")

	' Open database.
	Dim database
	Set database = _
	 installer.OpenDatabase(databasePath, msiOpenDatabaseModeTransact)

	' Apply fix transform.
	database.ApplyTransform importDir & "Fix.mst", 0

	Dim query, view

	' Fix File table: add a default value for Language if not set.
	query = "UPDATE `File` SET `Language`=0 WHERE `Language` IS NULL"
	Set view = database.OpenView(query)
	view.Execute

	' Add language-specific transforms as substorages.
	query = "SELECT `Name`,`Data` FROM _Storages"
	Set view = database.OpenView(query)
	AddRecord view, "fr-FR.mst", "1036"
	AddRecord view, "de-DE.mst", "1031"
	AddRecord view, "it-IT.mst", "1040"

	' Update summary information with language IDs.
	Dim sumInfo : Set sumInfo = database.SummaryInformation(1)
	sumInfo.Property(7) = "Intel;1033,1036,1031,1040"
	sumInfo.Persist

	' Commit changes.
	database.Commit
End Sub
